#!/bin/bash

source $(dirname $0)/common.sh $*

# Reconstruct
for rec in options/*.py ; do
  unset tag
  [[ $(basename ${rec} .py) =~ (.*)\.(.*) ]] && tag=".${BASH_REMATCH[2]}"
  JUGGLER_REC_FILE=${JUGGLER_REC_FILE/.root/${tag:-}.root} \
    /usr/bin/time -v \
    gaudirun.py ${JUGGLER_GAUDI_OPTIONS:-} ${rec}
  if [[ "$?" -ne "0" ]] ; then
    echo "ERROR running juggler"
    exit 1
  fi
done

#/usr/bin/time -v run_eicrecon_reco_flags.py ${JUGGLER_SIM_FILE} ${JUGGLER_REC_FILE_BASE} -Pplugins=janadot
valgrind --tool=helgrind \
eicrecon -Pjana:warmup_timeout=6000 -Pjana:timeout=300 -Pplugins=dump_flags,janadot -Pdump_flags:python=all_flags_dump_from_run.py -Pjana:debug_plugin_loading=1 -Pjana:nevents=0 -Pacts:MaterialMap=calibrations/materials-map.cbor -Ppodio:output_file=${JUGGLER_REC_FILE_BASE}.tree.edm4eic.root -Phistsfile=${JUGGLER_REC_FILE_BASE}.ana.root ${JUGGLER_SIM_FILE} -PBEMC:EcalBarrelSciGlassRawHits:input_tag=EcalBarrelSciGlassHits -PBEMC:EcalBarrelSciGlassRawHits:capacityADC=16384 -PBEMC:EcalBarrelSciGlassRawHits:dynamicRangeADC=20000.0 -PBEMC:EcalBarrelSciGlassRawHits:energyResolutions=0.0,0.02,0.0 -PBEMC:EcalBarrelSciGlassRawHits:fieldRefNumbers= -PBEMC:EcalBarrelSciGlassRawHits:geoServiceName=ActsGeometryProvider -PBEMC:EcalBarrelSciGlassRawHits:pedestalMean=100 -PBEMC:EcalBarrelSciGlassRawHits:pedestalSigma=1 -PBEMC:EcalBarrelSciGlassRawHits:readoutClass= -PBEMC:EcalBarrelSciGlassRawHits:resolutionTDC=1e-11 -PBEMC:EcalBarrelSciGlassRawHits:scaleResponse=1 -PBEMC:EcalBarrelSciGlassRawHits:signalSumFields= -PBEMC:EcalBarrelSciGlassRawHits:timeResolution=0 -PBEMC:EcalBarrelSciGlassRecHits:input_tag=EcalBarrelSciGlassRawHits -PBEMC:EcalBarrelSciGlassRecHits:capacityADC=16384 -PBEMC:EcalBarrelSciGlassRecHits:dynamicRangeADC=20000.0 -PBEMC:EcalBarrelSciGlassRecHits:pedestalMean=100 -PBEMC:EcalBarrelSciGlassRecHits:pedestalSigma=1 -PBEMC:EcalBarrelSciGlassRecHits:resolutionTDC=1e-11 -PBEMC:EcalBarrelSciGlassRecHits:samplingFraction=0.98 -PBEMC:EcalBarrelSciGlassRecHits:thresholdFactor=3 -PBEMC:EcalBarrelSciGlassRecHits:thresholdValue=3 -PBEMC:EcalBarrelSciGlassProtoClusters:input_tag=EcalBarrelSciGlassRecHits -PBEMC:EcalBarrelSciGlassProtoClusters:dimScaledLocalDistXY=1.8,1.8 -PBEMC:EcalBarrelSciGlassProtoClusters:globalDistEtaPhi= -PBEMC:EcalBarrelSciGlassProtoClusters:globalDistRPhi= -PBEMC:EcalBarrelSciGlassProtoClusters:localDistXY= -PBEMC:EcalBarrelSciGlassProtoClusters:localDistXZ= -PBEMC:EcalBarrelSciGlassProtoClusters:localDistYZ= -PBEMC:EcalBarrelSciGlassProtoClusters:minClusterCenterEdep=30.0 -PBEMC:EcalBarrelSciGlassProtoClusters:minClusterHitEdep=1.0 -PBEMC:EcalBarrelSciGlassProtoClusters:sectorDist=50.0 -PBEMC:EcalBarrelSciGlassProtoClusters:splitCluster=0 -PBEMC:EcalBarrelSciGlassClusters:input_protoclust_tag=EcalBarrelSciGlassProtoClusters -PBEMC:EcalBarrelSciGlassClusters:depthCorrection=0 -PBEMC:EcalBarrelSciGlassClusters:enableEtaBounds=1 -PBEMC:EcalBarrelSciGlassClusters:energyWeight=log -PBEMC:EcalBarrelSciGlassClusters:input_simhit_tag=EcalBarrelSciGlassHits -PBEMC:EcalBarrelSciGlassClusters:logWeightBase=6.2 -PBEMC:EcalBarrelSciGlassClusters:moduleDimZName= -PBEMC:EcalBarrelSciGlassClusters:samplingFraction=1 -PBEMC:EcalBarrelSciGlassTruthClusters:input_protoclust_tag=EcalBarrelSciGlassTruthProtoClusters -PBEMC:EcalBarrelSciGlassTruthClusters:depthCorrection=0 -PBEMC:EcalBarrelSciGlassTruthClusters:enableEtaBounds=1 -PBEMC:EcalBarrelSciGlassTruthClusters:energyWeight=log -PBEMC:EcalBarrelSciGlassTruthClusters:input_simhit_tag=EcalBarrelSciGlassHits -PBEMC:EcalBarrelSciGlassTruthClusters:logWeightBase=6.2 -PBEMC:EcalBarrelSciGlassTruthClusters:moduleDimZName= -PBEMC:EcalBarrelSciGlassTruthClusters:samplingFraction=1 -PBEMC:EcalBarrelImagingRawHits:input_tag=EcalBarrelImagingHits -PBEMC:EcalBarrelImagingRawHits:energyResolutions=0.0,0.02,0.0 -PBEMC:EcalBarrelImagingRawHits:timeResolution=0.0*ns -PBEMC:EcalBarrelImagingRawHits:capacityADC=8192 -PBEMC:EcalBarrelImagingRawHits:dynamicRangeADC=3.0 -PBEMC:EcalBarrelImagingRawHits:pedestalMean=100 -PBEMC:EcalBarrelImagingRawHits:pedestalSigma=14 -PBEMC:EcalBarrelImagingRawHits:resolutionTDC=10*picosecond -PBEMC:EcalBarrelImagingRawHits:scaleResponse=1.0 -PBEMC:EcalBarrelImagingRawHits:signalSumFields= -PBEMC:EcalBarrelImagingRawHits:fieldRefNumbers= -PBEMC:EcalBarrelImagingRawHits:geoServiceName=ActsGeometryProvider -PBEMC:EcalBarrelImagingRawHits:readoutClass=pixel -PBEMC:EcalBarrelScFiRawHits:input_tag=EcalBarrelScFiHits -PBEMC:EcalBarrelScFiRawHits:energyResolutions= -PBEMC:EcalBarrelScFiRawHits:timeResolution=0*ns -PBEMC:EcalBarrelScFiRawHits:capacityADC=16384 -PBEMC:EcalBarrelScFiRawHits:dynamicRangeADC=750.0 -PBEMC:EcalBarrelScFiRawHits:pedestalMean=20 -PBEMC:EcalBarrelScFiRawHits:pedestalSigma=0.3 -PBEMC:EcalBarrelScFiRawHits:resolutionTDC=10*picosecond -PBEMC:EcalBarrelScFiRawHits:scaleResponse=1.0 -PBEMC:EcalBarrelScFiRawHits:signalSumFields= -PBEMC:EcalBarrelScFiRawHits:fieldRefNumbers= -PBEMC:EcalBarrelScFiRawHits:geoServiceName=ActsGeometryProvider -PBEMC:EcalBarrelScFiRawHits:readoutClass=light_guide -PBEMC:EcalBarrelImagingRecHits:input_tag=EcalBarrelImagingRawHits -PBEMC:EcalBarrelImagingRecHits:layerField=layer -PBEMC:EcalBarrelImagingRecHits:sectorField=module -PBEMC:EcalBarrelImagingRecHits:capacityADC=8192 -PBEMC:EcalBarrelImagingRecHits:pedestalMean=100 -PBEMC:EcalBarrelImagingRecHits:dynamicRangeADC=3.0 -PBEMC:EcalBarrelImagingRecHits:pedSigmaADC=14 -PBEMC:EcalBarrelImagingRecHits:thresholdFactor=3.0 -PBEMC:EcalBarrelImagingRecHits:samplingFraction=0.005 -PBEMC:EcalBarrelScFiRecHits:input_tag=EcalBarrelScFiRawHits -PBEMC:EcalBarrelScFiRecHits:capacityADC=16384 -PBEMC:EcalBarrelScFiRecHits:dynamicRangeADC=750.0 -PBEMC:EcalBarrelScFiRecHits:pedestalMean=20 -PBEMC:EcalBarrelScFiRecHits:pedestalSigma=0.3 -PBEMC:EcalBarrelScFiRecHits:resolutionTDC=10*picosecond -PBEMC:EcalBarrelScFiRecHits:thresholdFactor=5.0 -PBEMC:EcalBarrelScFiRecHits:thresholdValue=0.0 -PBEMC:EcalBarrelScFiRecHits:samplingFraction=0.125 -PBEMC:EcalBarrelscFiMergedHits:input_tag=EcalBarrelScFiRecHits -PBEMC:EcalBarrelscFiMergedHits:fields=fiber,z -PBEMC:EcalBarrelscFiMergedHits:refs=1,1 -PBEMC:EcalBarrelImagingProtoClusters:input_tag=EcalBarrelImagingRecHits -PBEMC:EcalBarrelImagingProtoClusters::localDistXY=2.0,2.0 -PBEMC:EcalBarrelImagingProtoClusters::layerDistEtaPhi=0.01,0.01 -PBEMC:EcalBarrelImagingProtoClusters::neighbourLayersRange=2.0 -PBEMC:EcalBarrelImagingProtoClusters::sectorDist=30.0 -PBEMC:EcalBarrelImagingProtoClusters::minClusterHitEdep=0. -PBEMC:EcalBarrelImagingProtoClusters::minClusterCenterEdep=0. -PBEMC:EcalBarrelImagingProtoClusters::minClusterEdep=0.5 -PBEMC:EcalBarrelImagingProtoClusters::minClusterNhits=5 -PBEMC:EcalBarrelScFiProtoClusters:input_tag=EcalBarrelScFiMergedHits -PBEMC:EcalBarrelScFiProtoClusters:splitCluster=false -PBEMC:EcalBarrelScFiProtoClusters:minClusterHitEdep=1.0 -PBEMC:EcalBarrelScFiProtoClusters:minClusterCenterEdep=10.0 -PBEMC:EcalBarrelScFiProtoClusters:sectorDist=50.0 -PBEMC:EcalBarrelScFiProtoClusters:localDistXY= -PBEMC:EcalBarrelScFiProtoClusters:localDistXZ=30.0,30.0 -PBEMC:EcalBarrelScFiProtoClusters:localDistYZ= -PBEMC:EcalBarrelScFiProtoClusters:globalDistRPhi= -PBEMC:EcalBarrelScFiProtoClusters:globalDistEtaPhi= -PBEMC:EcalBarrelScFiProtoClusters:dimScaledLocalDistXY= -PBEMC:EcalBarrelImagingClusters:input_protoclust_tag=EcalBarrelImagingProtoClusters -PBEMC:EcalBarrelImagingClusters:trackStopLayer=6 -PBEMC:EcalBarrelScFiClusters:input_protoclust_tag=EcalBarrelScFiProtoClusters -PBEMC:EcalBarrelScFiClusters:samplingFraction=1.0 -PBEMC:EcalBarrelScFiClusters:logWeightBase=6.2 -PBEMC:EcalBarrelScFiClusters:depthCorrection=0.0 -PBEMC:EcalBarrelScFiClusters:input_simhit_tag=EcalBarrelScFiHits -PBEMC:EcalBarrelScFiClusters:energyWeight=log -PBEMC:EcalBarrelScFiClusters:moduleDimZName= -PBEMC:EcalBarrelScFiClusters:enableEtaBounds=false -PBEMC:EcalBarrelImagingMergedClusters:inputMCParticles_tag=MCParticles -PBEMC:EcalBarrelImagingMergedClusters:energyClusters_tag=EcalBarrelScFiClusters -PBEMC:EcalBarrelImagingMergedClusters:energyAssociation_tag=EcalBarrelScFiClusterAssociations -PBEMC:EcalBarrelImagingMergedClusters:positionClusters_tag=EcalBarrelImagingClusters -PBEMC:EcalBarrelImagingMergedClusters:positionAssociations_tag=EcalBarrelImagingClusterAssociations -PEEMC:EcalEndcapNRawHits:capacityADC=16384 -PEEMC:EcalEndcapNRawHits:dynamicRangeADC=20000.0 -PEEMC:EcalEndcapNRawHits:energyResolutions=0.0,0.02,0.0 -PEEMC:EcalEndcapNRawHits:fieldRefNumbers= -PEEMC:EcalEndcapNRawHits:geoServiceName=ActsGeometryProvider -PEEMC:EcalEndcapNRawHits:pedestalMean=100 -PEEMC:EcalEndcapNRawHits:pedestalSigma=1 -PEEMC:EcalEndcapNRawHits:readoutClass= -PEEMC:EcalEndcapNRawHits:resolutionTDC=1e-11 -PEEMC:EcalEndcapNRawHits:scaleResponse=1 -PEEMC:EcalEndcapNRawHits:signalSumFields= -PEEMC:EcalEndcapNRawHits:timeResolution=0 -PEEMC:EcalEndcapNRecHits:capacityADC=16384 -PEEMC:EcalEndcapNRecHits:dynamicRangeADC=20000.0 -PEEMC:EcalEndcapNRecHits:geoServiceName=geoServiceName -PEEMC:EcalEndcapNRecHits:layerField= -PEEMC:EcalEndcapNRecHits:localDetElement= -PEEMC:EcalEndcapNRecHits:localDetFields= -PEEMC:EcalEndcapNRecHits:pedestalMean=100 -PEEMC:EcalEndcapNRecHits:pedestalSigma=1 -PEEMC:EcalEndcapNRecHits:readout=EcalEndcapNHits -PEEMC:EcalEndcapNRecHits:resolutionTDC=1e-11 -PEEMC:EcalEndcapNRecHits:samplingFraction=0.998 -PEEMC:EcalEndcapNRecHits:sectorField=sector -PEEMC:EcalEndcapNRecHits:thresholdFactor=4 -PEEMC:EcalEndcapNRecHits:thresholdValue=3 -PEEMC:EcalEndcapNIslandProtoClusters:dimScaledLocalDistXY=1.8,1.8 -PEEMC:EcalEndcapNIslandProtoClusters:globalDistEtaPhi= -PEEMC:EcalEndcapNIslandProtoClusters:globalDistRPhi= -PEEMC:EcalEndcapNIslandProtoClusters:localDistXY= -PEEMC:EcalEndcapNIslandProtoClusters:localDistXZ= -PEEMC:EcalEndcapNIslandProtoClusters:localDistYZ= -PEEMC:EcalEndcapNIslandProtoClusters:minClusterCenterEdep=1.0 -PEEMC:EcalEndcapNIslandProtoClusters:minClusterHitEdep=30.0 -PEEMC:EcalEndcapNIslandProtoClusters:sectorDist=50.0 -PEEMC:EcalEndcapNIslandProtoClusters:splitCluster=0 -PEEMC:EcalEndcapNTruthClusters:depthCorrection=0 -PEEMC:EcalEndcapNTruthClusters:enableEtaBounds=0 -PEEMC:EcalEndcapNTruthClusters:energyWeight=log -PEEMC:EcalEndcapNTruthClusters:logWeightBase=4.6 -PEEMC:EcalEndcapNTruthClusters:moduleDimZName= -PEEMC:EcalEndcapNTruthClusters:samplingFraction=0.03 -PEEMC:EcalEndcapNClusters:input_protoclust_tag=EcalEndcapNIslandProtoClusters -PEEMC:EcalEndcapNClusters:samplingFraction=1 -PEEMC:EcalEndcapNClusters:logWeightBase=3.6 -PEEMC:EcalEndcapNClusters:depthCorrection=0 -PEEMC:EcalEndcapNClusters:energyWeight=log -PEEMC:EcalEndcapNClusters:moduleDimZName= -PEEMC:EcalEndcapNClusters:enableEtaBounds=0 -PEEMC:EcalEndcapPRawHits:capacityADC=16384 -PEEMC:EcalEndcapPRawHits:dynamicRangeADC=3000.0 -PEEMC:EcalEndcapPRawHits:energyResolutions=0.00316,0.0015,0.0 -PEEMC:EcalEndcapPRawHits:fieldRefNumbers=1,1 -PEEMC:EcalEndcapPRawHits:geoServiceName=ActsGeometryProvider -PEEMC:EcalEndcapPRawHits:pedestalMean=100 -PEEMC:EcalEndcapPRawHits:pedestalSigma=0.7 -PEEMC:EcalEndcapPRawHits:readoutClass= -PEEMC:EcalEndcapPRawHits:resolutionTDC=1e-11 -PEEMC:EcalEndcapPRawHits:scaleResponse=0.03 -PEEMC:EcalEndcapPRawHits:signalSumFields= -PEEMC:EcalEndcapPRawHits:timeResolution=0 -PEEMC:EcalEndcapPRecHits:capacityADC=16384 -PEEMC:EcalEndcapPRecHits:dynamicRangeADC=3000.0 -PEEMC:EcalEndcapPRecHits:geoServiceName=geoServiceName -PEEMC:EcalEndcapPRecHits:layerField= -PEEMC:EcalEndcapPRecHits:localDetElement= -PEEMC:EcalEndcapPRecHits:localDetFields= -PEEMC:EcalEndcapPRecHits:pedestalMean=100 -PEEMC:EcalEndcapPRecHits:pedestalSigma=0.7 -PEEMC:EcalEndcapPRecHits:readout=EcalEndcapPHits -PEEMC:EcalEndcapPRecHits:resolutionTDC=1e-11 -PEEMC:EcalEndcapPRecHits:samplingFraction=0.03 -PEEMC:EcalEndcapPRecHits:sectorField= -PEEMC:EcalEndcapPRecHits:thresholdFactor=5.0 -PEEMC:EcalEndcapPRecHits:thresholdValue=2 -PEEMC:EcalEndcapPIslandProtoClusters:dimScaledLocalDistXY=1.5,1.5 -PEEMC:EcalEndcapPIslandProtoClusters:globalDistEtaPhi= -PEEMC:EcalEndcapPIslandProtoClusters:globalDistRPhi= -PEEMC:EcalEndcapPIslandProtoClusters:localDistXY=10,10 -PEEMC:EcalEndcapPIslandProtoClusters:localDistXZ= -PEEMC:EcalEndcapPIslandProtoClusters:localDistYZ= -PEEMC:EcalEndcapPIslandProtoClusters:minClusterCenterEdep=10.0 -PEEMC:EcalEndcapPIslandProtoClusters:minClusterHitEdep=0 -PEEMC:EcalEndcapPIslandProtoClusters:sectorDist=5 -PEEMC:EcalEndcapPIslandProtoClusters:splitCluster=0 -PEEMC:EcalEndcapPTruthClusters:depthCorrection=0 -PEEMC:EcalEndcapPTruthClusters:enableEtaBounds=1 -PEEMC:EcalEndcapPTruthClusters:energyWeight=log -PEEMC:EcalEndcapPTruthClusters:logWeightBase=6.2 -PEEMC:EcalEndcapPTruthClusters:moduleDimZName= -PEEMC:EcalEndcapPTruthClusters:samplingFraction=1 -PEEMC:EcalEndcapPClusters:input_protoclust_tag=EcalEndcapPIslandProtoClusters -PEEMC:EcalEndcapPClusters:samplingFraction=1 -PEEMC:EcalEndcapPClusters:logWeightBase=3.6 -PEEMC:EcalEndcapPClusters:depthCorrection=0 -PEEMC:EcalEndcapPClusters:energyWeight=log -PEEMC:EcalEndcapPClusters:moduleDimZName= -PEEMC:EcalEndcapPClusters:enableEtaBounds=0 -PEEMC:EcalEndcapPInsertRawHits:capacityADC=16384 -PEEMC:EcalEndcapPInsertRawHits:dynamicRangeADC=3000.0 -PEEMC:EcalEndcapPInsertRawHits:energyResolutions=0.00316,0.0015,0.0 -PEEMC:EcalEndcapPInsertRawHits:fieldRefNumbers=1,1 -PEEMC:EcalEndcapPInsertRawHits:geoServiceName=ActsGeometryProvider -PEEMC:EcalEndcapPInsertRawHits:pedestalMean=100 -PEEMC:EcalEndcapPInsertRawHits:pedestalSigma=0.7 -PEEMC:EcalEndcapPInsertRawHits:readoutClass= -PEEMC:EcalEndcapPInsertRawHits:resolutionTDC=1e-11 -PEEMC:EcalEndcapPInsertRawHits:scaleResponse=0.03 -PEEMC:EcalEndcapPInsertRawHits:signalSumFields= -PEEMC:EcalEndcapPInsertRawHits:timeResolution=0 -PEEMC:EcalEndcapPInsertRecHits:capacityADC=16384 -PEEMC:EcalEndcapPInsertRecHits:dynamicRangeADC=3000.0 -PEEMC:EcalEndcapPInsertRecHits:geoServiceName=geoServiceName -PEEMC:EcalEndcapPInsertRecHits:layerField= -PEEMC:EcalEndcapPInsertRecHits:localDetElement= -PEEMC:EcalEndcapPInsertRecHits:localDetFields= -PEEMC:EcalEndcapPInsertRecHits:pedestalMean=100 -PEEMC:EcalEndcapPInsertRecHits:pedestalSigma=0.7 -PEEMC:EcalEndcapPInsertRecHits:readout=EcalEndcapPInsertHits -PEEMC:EcalEndcapPInsertRecHits:resolutionTDC=1e-11 -PEEMC:EcalEndcapPInsertRecHits:samplingFraction=0.03 -PEEMC:EcalEndcapPInsertRecHits:sectorField= -PEEMC:EcalEndcapPInsertRecHits:thresholdFactor=5.0 -PEEMC:EcalEndcapPInsertRecHits:thresholdValue=2 -PEEMC:EcalEndcapPInsertIslandProtoClusters:dimScaledLocalDistXY=1.5,1.5 -PEEMC:EcalEndcapPInsertIslandProtoClusters:globalDistEtaPhi= -PEEMC:EcalEndcapPInsertIslandProtoClusters:globalDistRPhi= -PEEMC:EcalEndcapPInsertIslandProtoClusters:localDistXY=10,10 -PEEMC:EcalEndcapPInsertIslandProtoClusters:localDistXZ= -PEEMC:EcalEndcapPInsertIslandProtoClusters:localDistYZ= -PEEMC:EcalEndcapPInsertIslandProtoClusters:minClusterCenterEdep=10.0 -PEEMC:EcalEndcapPInsertIslandProtoClusters:minClusterHitEdep=0 -PEEMC:EcalEndcapPInsertIslandProtoClusters:sectorDist=5 -PEEMC:EcalEndcapPInsertIslandProtoClusters:splitCluster=0 -PEEMC:EcalEndcapPInsertTruthClusters:depthCorrection=0 -PEEMC:EcalEndcapPInsertTruthClusters:enableEtaBounds=1 -PEEMC:EcalEndcapPInsertTruthClusters:energyWeight=log -PEEMC:EcalEndcapPInsertTruthClusters:logWeightBase=6.2 -PEEMC:EcalEndcapPInsertTruthClusters:moduleDimZName= -PEEMC:EcalEndcapPInsertTruthClusters:samplingFraction=1 -PEEMC:EcalEndcapPInsertClusters:input_protoclust_tag=EcalEndcapPInsertIslandProtoClusters -PEEMC:EcalEndcapPInsertClusters:samplingFraction=1 -PEEMC:EcalEndcapPInsertClusters:logWeightBase=3.6 -PEEMC:EcalEndcapPInsertClusters:depthCorrection=0 -PEEMC:EcalEndcapPInsertClusters:energyWeight=log -PEEMC:EcalEndcapPInsertClusters:moduleDimZName= -PEEMC:EcalEndcapPInsertClusters:enableEtaBounds=0 -PB0ECAL:B0ECalRawHits:capacityADC=16384 -PB0ECAL:B0ECalRawHits:dynamicRangeADC=20000.0 -PB0ECAL:B0ECalRawHits:energyResolutions=0.0,0.02,0.0 -PB0ECAL:B0ECalRawHits:fieldRefNumbers= -PB0ECAL:B0ECalRawHits:geoServiceName= -PB0ECAL:B0ECalRawHits:pedestalMean=100 -PB0ECAL:B0ECalRawHits:pedestalSigma=1 -PB0ECAL:B0ECalRawHits:readoutClass= -PB0ECAL:B0ECalRawHits:resolutionTDC=1e-11 -PB0ECAL:B0ECalRawHits:scaleResponse=1 -PB0ECAL:B0ECalRawHits:signalSumFields= -PB0ECAL:B0ECalRawHits:timeResolution=0 -PB0ECAL:B0ECalRecHits:capacityADC=16384 -PB0ECAL:B0ECalRecHits:dynamicRangeADC=20000.0 -PB0ECAL:B0ECalRecHits:geoServiceName= -PB0ECAL:B0ECalRecHits:layerField= -PB0ECAL:B0ECalRecHits:localDetElement= -PB0ECAL:B0ECalRecHits:localDetFields= -PB0ECAL:B0ECalRecHits:pedestalMean=100 -PB0ECAL:B0ECalRecHits:pedestalSigma=1 -PB0ECAL:B0ECalRecHits:readout=B0ECalHits -PB0ECAL:B0ECalRecHits:resolutionTDC=1e-11 -PB0ECAL:B0ECalRecHits:samplingFraction=0.998 -PB0ECAL:B0ECalRecHits:sectorField=sector -PB0ECAL:B0ECalRecHits:thresholdFactor=4 -PB0ECAL:B0ECalRecHits:thresholdValue=3 -PB0ECAL:B0ECalIslandProtoClusters:input_tag=B0ECalRecHits -PB0ECAL:B0ECalIslandProtoClusters:splitCluster=0 -PB0ECAL:B0ECalIslandProtoClusters:minClusterHitEdep=30.0 -PB0ECAL:B0ECalIslandProtoClusters:minClusterCenterEdep=1.0 -PB0ECAL:B0ECalIslandProtoClusters:sectorDist=50.0 -PB0ECAL:B0ECalIslandProtoClusters:localDistXY= -PB0ECAL:B0ECalIslandProtoClusters:localDistXZ= -PB0ECAL:B0ECalIslandProtoClusters:localDistYZ= -PB0ECAL:B0ECalIslandProtoClusters:globalDistRPhi= -PB0ECAL:B0ECalIslandProtoClusters:globalDistEtaPhi= -PB0ECAL:B0ECalIslandProtoClusters:dimScaledLocalDistXY=1.8,1.8 -PB0ECAL:B0ECalClusters:input_protoclust_tag=B0ECalIslandProtoClusters -PB0ECAL:B0ECalClusters:samplingFraction=1 -PB0ECAL:B0ECalClusters:logWeightBase=3.6 -PB0ECAL:B0ECalClusters:depthCorrection=0 -PB0ECAL:B0ECalClusters:energyWeight=log -PB0ECAL:B0ECalClusters:moduleDimZName= -PB0ECAL:B0ECalClusters:enableEtaBounds=0 -PHCAL:HcalBarrelRawHits:capacityADC=256 -PHCAL:HcalBarrelRawHits:dynamicRangeADC=50.0 -PHCAL:HcalBarrelRawHits:energyResolutions= -PHCAL:HcalBarrelRawHits:fieldRefNumbers=1,0 -PHCAL:HcalBarrelRawHits:geoServiceName=ActsGeometryProvider -PHCAL:HcalBarrelRawHits:pedestalMean=10 -PHCAL:HcalBarrelRawHits:pedestalSigma=2 -PHCAL:HcalBarrelRawHits:readoutClass= -PHCAL:HcalBarrelRawHits:resolutionTDC=1e-9 -PHCAL:HcalBarrelRawHits:scaleResponse=1 -PHCAL:HcalBarrelRawHits:signalSumFields= -PHCAL:HcalBarrelRawHits:timeResolution=0 -PHCAL:HcalBarrelRecHits:capacityADC=256 -PHCAL:HcalBarrelRecHits:dynamicRangeADC=20.0 -PHCAL:HcalBarrelRecHits:geoServiceName=geoServiceName -PHCAL:HcalBarrelRecHits:layerField=tower -PHCAL:HcalBarrelRecHits:localDetElement= -PHCAL:HcalBarrelRecHits:localDetFields= -PHCAL:HcalBarrelRecHits:pedestalMean=20 -PHCAL:HcalBarrelRecHits:pedestalSigma=0.3 -PHCAL:HcalBarrelRecHits:readout=HcalBarrelHits -PHCAL:HcalBarrelRecHits:resolutionTDC=1e-11 -PHCAL:HcalBarrelRecHits:samplingFraction=0.033 -PHCAL:HcalBarrelRecHits:sectorField=sector -PHCAL:HcalBarrelRecHits:thresholdFactor=5 -PHCAL:HcalBarrelRecHits:thresholdValue=1 -PHCAL:HcalBarrelClusters:depthCorrection=0 -PHCAL:HcalBarrelClusters:enableEtaBounds=0 -PHCAL:HcalBarrelClusters:energyWeight=log -PHCAL:HcalBarrelClusters:input_protoclust_tag=HcalBarrelIslandProtoClusters -PHCAL:HcalBarrelClusters:input_simhit_tag=HcalBarrelHits -PHCAL:HcalBarrelClusters:logWeightBase=6.2 -PHCAL:HcalBarrelClusters:moduleDimZName= -PHCAL:HcalBarrelClusters:samplingFraction=1 -PHCAL:HcalBarrelIslandProtoClusters:dimScaledLocalDistXY=5,5 -PHCAL:HcalBarrelIslandProtoClusters:globalDistEtaPhi= -PHCAL:HcalBarrelIslandProtoClusters:globalDistRPhi= -PHCAL:HcalBarrelIslandProtoClusters:localDistXY=150,150 -PHCAL:HcalBarrelIslandProtoClusters:localDistXZ= -PHCAL:HcalBarrelIslandProtoClusters:localDistYZ= -PHCAL:HcalBarrelIslandProtoClusters:minClusterCenterEdep=0.003 -PHCAL:HcalBarrelIslandProtoClusters:minClusterHitEdep=30.0 -PHCAL:HcalBarrelIslandProtoClusters:sectorDist=5 -PHCAL:HcalBarrelIslandProtoClusters:splitCluster=0 -PHCAL:HcalBarrelMergedHits:fields=tower,tile -PHCAL:HcalBarrelMergedHits:input_tag=HcalBarrelRecHits -PHCAL:HcalBarrelMergedHits:refs=1,0 -PHCAL:HcalBarrelTruthClusters:depthCorrection=0 -PHCAL:HcalBarrelTruthClusters:enableEtaBounds=0 -PHCAL:HcalBarrelTruthClusters:energyWeight=log -PHCAL:HcalBarrelTruthClusters:input_protoclust_tag=HcalBarrelTruthProtoClusters -PHCAL:HcalBarrelTruthClusters:input_simhit_tag=HcalBarrelHits -PHCAL:HcalBarrelTruthClusters:logWeightBase=6.2 -PHCAL:HcalBarrelTruthClusters:moduleDimZName= -PHCAL:HcalBarrelTruthClusters:samplingFraction=1 -PHCAL:HcalEndcapNRawHits:capacityADC=1024 -PHCAL:HcalEndcapNRawHits:dynamicRangeADC=3.6 -PHCAL:HcalEndcapNRawHits:energyResolutions= -PHCAL:HcalEndcapNRawHits:fieldRefNumbers= -PHCAL:HcalEndcapNRawHits:geoServiceName=ActsGeometryProvider -PHCAL:HcalEndcapNRawHits:pedestalMean=20 -PHCAL:HcalEndcapNRawHits:pedestalSigma=0.3 -PHCAL:HcalEndcapNRawHits:readoutClass= -PHCAL:HcalEndcapNRawHits:resolutionTDC=1e-11 -PHCAL:HcalEndcapNRawHits:scaleResponse=1 -PHCAL:HcalEndcapNRawHits:signalSumFields= -PHCAL:HcalEndcapNRawHits:timeResolution=0 -PHCAL:HcalEndcapNRecHits:capacityADC=1024 -PHCAL:HcalEndcapNRecHits:dynamicRangeADC=3.6 -PHCAL:HcalEndcapNRecHits:geoServiceName=geoServiceName -PHCAL:HcalEndcapNRecHits:layerField= -PHCAL:HcalEndcapNRecHits:localDetElement= -PHCAL:HcalEndcapNRecHits:localDetFields= -PHCAL:HcalEndcapNRecHits:pedestalMean=20 -PHCAL:HcalEndcapNRecHits:pedestalSigma=0.3 -PHCAL:HcalEndcapNRecHits:readout=HcalEndcapNHits -PHCAL:HcalEndcapNRecHits:resolutionTDC=1e-11 -PHCAL:HcalEndcapNRecHits:samplingFraction=0.998 -PHCAL:HcalEndcapNRecHits:sectorField= -PHCAL:HcalEndcapNRecHits:thresholdFactor=4 -PHCAL:HcalEndcapNRecHits:thresholdValue=1 -PHCAL:HcalEndcapNMergedHits:input_tag=HcalEndcapNRecHits -PHCAL:HcalEndcapNMergedHits:readout=HcalEndcapNHits -PHCAL:HcalEndcapNMergedHits:fields=layer,slice -PHCAL:HcalEndcapNMergedHits:refs=1,0 -PHCAL:HcalEndcapNIslandProtoClusters:dimScaledLocalDistXY=1.5,1.5 -PHCAL:HcalEndcapNIslandProtoClusters:globalDistEtaPhi= -PHCAL:HcalEndcapNIslandProtoClusters:globalDistRPhi= -PHCAL:HcalEndcapNIslandProtoClusters:localDistXY=150,150 -PHCAL:HcalEndcapNIslandProtoClusters:localDistXZ= -PHCAL:HcalEndcapNIslandProtoClusters:localDistYZ= -PHCAL:HcalEndcapNIslandProtoClusters:minClusterCenterEdep=30.0 -PHCAL:HcalEndcapNIslandProtoClusters:minClusterHitEdep=0 -PHCAL:HcalEndcapNIslandProtoClusters:sectorDist=5 -PHCAL:HcalEndcapNIslandProtoClusters:splitCluster=1 -PHCAL:HcalEndcapNTruthClusters:depthCorrection=0 -PHCAL:HcalEndcapNTruthClusters:enableEtaBounds=0 -PHCAL:HcalEndcapNTruthClusters:energyWeight=log -PHCAL:HcalEndcapNTruthClusters:input_protoclust_tag=HcalEndcapNTruthProtoClusters -PHCAL:HcalEndcapNTruthClusters:input_simhit_tag=HcalEndcapNHits -PHCAL:HcalEndcapNTruthClusters:logWeightBase=6.2 -PHCAL:HcalEndcapNTruthClusters:moduleDimZName= -PHCAL:HcalEndcapNTruthClusters:samplingFraction=1 -PHCAL:HcalEndcapNClusters:depthCorrection=0 -PHCAL:HcalEndcapNClusters:enableEtaBounds=0 -PHCAL:HcalEndcapNClusters:energyWeight=log -PHCAL:HcalEndcapNClusters:input_protoclust_tag=HcalEndcapNIslandProtoClusters -PHCAL:HcalEndcapNClusters:input_simhit_tag=HcalEndcapNHits -PHCAL:HcalEndcapNClusters:logWeightBase=6.2 -PHCAL:HcalEndcapNClusters:moduleDimZName= -PHCAL:HcalEndcapNClusters:samplingFraction=1 -PHCAL:HcalEndcapPRawHits:capacityADC=1024 -PHCAL:HcalEndcapPRawHits:dynamicRangeADC=3600.0 -PHCAL:HcalEndcapPRawHits:energyResolutions= -PHCAL:HcalEndcapPRawHits:fieldRefNumbers= -PHCAL:HcalEndcapPRawHits:geoServiceName=ActsGeometryProvider -PHCAL:HcalEndcapPRawHits:pedestalMean=20 -PHCAL:HcalEndcapPRawHits:pedestalSigma=0.8 -PHCAL:HcalEndcapPRawHits:readoutClass= -PHCAL:HcalEndcapPRawHits:resolutionTDC=1e-11 -PHCAL:HcalEndcapPRawHits:scaleResponse=1 -PHCAL:HcalEndcapPRawHits:signalSumFields= -PHCAL:HcalEndcapPRawHits:timeResolution=0 -PHCAL:HcalEndcapPRecHits:capacityADC=1024 -PHCAL:HcalEndcapPRecHits:dynamicRangeADC=3600.0 -PHCAL:HcalEndcapPRecHits:geoServiceName=geoServiceName -PHCAL:HcalEndcapPRecHits:layerField= -PHCAL:HcalEndcapPRecHits:localDetElement= -PHCAL:HcalEndcapPRecHits:localDetFields= -PHCAL:HcalEndcapPRecHits:pedestalMean=20 -PHCAL:HcalEndcapPRecHits:pedestalSigma=0.8 -PHCAL:HcalEndcapPRecHits:readout=HcalEndcapPHits -PHCAL:HcalEndcapPRecHits:resolutionTDC=1e-11 -PHCAL:HcalEndcapPRecHits:samplingFraction=0.025 -PHCAL:HcalEndcapPRecHits:sectorField= -PHCAL:HcalEndcapPRecHits:thresholdFactor=5 -PHCAL:HcalEndcapPRecHits:thresholdValue=3 -PHCAL:HcalEndcapPMergedHits:input_tag=HcalEndcapPRecHits -PHCAL:HcalEndcapPMergedHits:readout=HcalEndcapPHits -PHCAL:HcalEndcapPMergedHits:fields=layer,slice -PHCAL:HcalEndcapPMergedHits:refs=1,0 -PHCAL:HcalEndcapPIslandProtoClusters:dimScaledLocalDistXY=1.5,1.5 -PHCAL:HcalEndcapPIslandProtoClusters:globalDistEtaPhi= -PHCAL:HcalEndcapPIslandProtoClusters:globalDistRPhi= -PHCAL:HcalEndcapPIslandProtoClusters:localDistXY=150,150 -PHCAL:HcalEndcapPIslandProtoClusters:localDistXZ= -PHCAL:HcalEndcapPIslandProtoClusters:localDistYZ= -PHCAL:HcalEndcapPIslandProtoClusters:minClusterCenterEdep=30.0 -PHCAL:HcalEndcapPIslandProtoClusters:minClusterHitEdep=0 -PHCAL:HcalEndcapPIslandProtoClusters:sectorDist=5 -PHCAL:HcalEndcapPIslandProtoClusters:splitCluster=1 -PHCAL:HcalEndcapPTruthClusters:depthCorrection=0 -PHCAL:HcalEndcapPTruthClusters:enableEtaBounds=0 -PHCAL:HcalEndcapPTruthClusters:energyWeight=log -PHCAL:HcalEndcapPTruthClusters:input_protoclust_tag=HcalEndcapPTruthProtoClusters -PHCAL:HcalEndcapPTruthClusters:input_simhit_tag=HcalEndcapPHits -PHCAL:HcalEndcapPTruthClusters:logWeightBase=6.2 -PHCAL:HcalEndcapPTruthClusters:moduleDimZName= -PHCAL:HcalEndcapPTruthClusters:samplingFraction=0.025 -PHCAL:HcalEndcapPClusters:depthCorrection=0 -PHCAL:HcalEndcapPClusters:enableEtaBounds=0 -PHCAL:HcalEndcapPClusters:energyWeight=log -PHCAL:HcalEndcapPClusters:input_protoclust_tag=HcalEndcapPIslandProtoClusters -PHCAL:HcalEndcapPClusters:input_simhit_tag=HcalEndcapPHits -PHCAL:HcalEndcapPClusters:logWeightBase=6.2 -PHCAL:HcalEndcapPClusters:moduleDimZName= -PHCAL:HcalEndcapPClusters:samplingFraction=0.025 -PHCAL:HcalEndcapPInsertRawHits:capacityADC=32768 -PHCAL:HcalEndcapPInsertRawHits:dynamicRangeADC=200.0 -PHCAL:HcalEndcapPInsertRawHits:energyResolutions= -PHCAL:HcalEndcapPInsertRawHits:fieldRefNumbers= -PHCAL:HcalEndcapPInsertRawHits:geoServiceName=ActsGeometryProvider -PHCAL:HcalEndcapPInsertRawHits:pedestalMean=400 -PHCAL:HcalEndcapPInsertRawHits:pedestalSigma=10 -PHCAL:HcalEndcapPInsertRawHits:readoutClass= -PHCAL:HcalEndcapPInsertRawHits:resolutionTDC=1e-11 -PHCAL:HcalEndcapPInsertRawHits:scaleResponse=1 -PHCAL:HcalEndcapPInsertRawHits:signalSumFields= -PHCAL:HcalEndcapPInsertRawHits:timeResolution=0 -PHCAL:HcalEndcapPInsertRecHits:capacityADC=32768 -PHCAL:HcalEndcapPInsertRecHits:dynamicRangeADC=200.0 -PHCAL:HcalEndcapPInsertRecHits:geoServiceName=geoServiceName -PHCAL:HcalEndcapPInsertRecHits:layerField= -PHCAL:HcalEndcapPInsertRecHits:localDetElement= -PHCAL:HcalEndcapPInsertRecHits:localDetFields= -PHCAL:HcalEndcapPInsertRecHits:pedestalMean=400 -PHCAL:HcalEndcapPInsertRecHits:pedestalSigma=10 -PHCAL:HcalEndcapPInsertRecHits:readout=HcalEndcapPInsertHits -PHCAL:HcalEndcapPInsertRecHits:resolutionTDC=1e-11 -PHCAL:HcalEndcapPInsertRecHits:samplingFraction=0.998 -PHCAL:HcalEndcapPInsertRecHits:sectorField= -PHCAL:HcalEndcapPInsertRecHits:thresholdFactor=4 -PHCAL:HcalEndcapPInsertRecHits:thresholdValue=0 -PHCAL:HcalEndcapPInsertIslandProtoClusters:dimScaledLocalDistXY=1.5,1.5 -PHCAL:HcalEndcapPInsertIslandProtoClusters:globalDistEtaPhi= -PHCAL:HcalEndcapPInsertIslandProtoClusters:globalDistRPhi= -PHCAL:HcalEndcapPInsertIslandProtoClusters:localDistXY= -PHCAL:HcalEndcapPInsertIslandProtoClusters:localDistXZ= -PHCAL:HcalEndcapPInsertIslandProtoClusters:localDistYZ= -PHCAL:HcalEndcapPInsertIslandProtoClusters:minClusterCenterEdep=0.03 -PHCAL:HcalEndcapPInsertIslandProtoClusters:minClusterHitEdep=0 -PHCAL:HcalEndcapPInsertIslandProtoClusters:sectorDist=5 -PHCAL:HcalEndcapPInsertIslandProtoClusters:splitCluster=1 -PHCAL:HcalEndcapPInsertTruthClusters:depthCorrection=0 -PHCAL:HcalEndcapPInsertTruthClusters:enableEtaBounds=0 -PHCAL:HcalEndcapPInsertTruthClusters:energyWeight=log -PHCAL:HcalEndcapPInsertTruthClusters:input_protoclust_tag=HcalEndcapPInsertTruthProtoClusters -PHCAL:HcalEndcapPInsertTruthClusters:input_simhit_tag=HcalEndcapPInsertHits -PHCAL:HcalEndcapPInsertTruthClusters:logWeightBase=6.2 -PHCAL:HcalEndcapPInsertTruthClusters:moduleDimZName= -PHCAL:HcalEndcapPInsertTruthClusters:samplingFraction=1 -PHCAL:HcalEndcapPInsertClusters:depthCorrection=0 -PHCAL:HcalEndcapPInsertClusters:enableEtaBounds=0 -PHCAL:HcalEndcapPInsertClusters:energyWeight=log -PHCAL:HcalEndcapPInsertClusters:input_protoclust_tag=HcalEndcapPInsertIslandProtoClusters -PHCAL:HcalEndcapPInsertClusters:input_simhit_tag=HcalEndcapPInsertHits -PHCAL:HcalEndcapPInsertClusters:logWeightBase=6.2 -PHCAL:HcalEndcapPInsertClusters:moduleDimZName= -PHCAL:HcalEndcapPInsertClusters:samplingFraction=1 -PZDC:ZDCEcalRawHits:capacityADC=8096 -PZDC:ZDCEcalRawHits:dynamicRangeADC=0.1 -PZDC:ZDCEcalRawHits:energyResolutions= -PZDC:ZDCEcalRawHits:fieldRefNumbers= -PZDC:ZDCEcalRawHits:geoServiceName=ActsGeometryProvider -PZDC:ZDCEcalRawHits:pedestalMean=400 -PZDC:ZDCEcalRawHits:pedestalSigma=3.2 -PZDC:ZDCEcalRawHits:readoutClass= -PZDC:ZDCEcalRawHits:resolutionTDC=1e-11 -PZDC:ZDCEcalRawHits:scaleResponse=1 -PZDC:ZDCEcalRawHits:signalSumFields= -PZDC:ZDCEcalRawHits:timeResolution=0 -PZDC:ZDCEcalRecHits:capacityADC=8096 -PZDC:ZDCEcalRecHits:dynamicRangeADC=0.1 -PZDC:ZDCEcalRecHits:geoServiceName=geoServiceName -PZDC:ZDCEcalRecHits:layerField= -PZDC:ZDCEcalRecHits:localDetElement= -PZDC:ZDCEcalRecHits:localDetFields= -PZDC:ZDCEcalRecHits:pedestalMean=400 -PZDC:ZDCEcalRecHits:pedestalSigma=3.2 -PZDC:ZDCEcalRecHits:readout=ZDCEcalHits -PZDC:ZDCEcalRecHits:resolutionTDC=1e-11 -PZDC:ZDCEcalRecHits:samplingFraction=1 -PZDC:ZDCEcalRecHits:sectorField= -PZDC:ZDCEcalRecHits:thresholdFactor=4 -PZDC:ZDCEcalRecHits:thresholdValue=0 -PZDC:ZDCEcalClusters:depthCorrection=0 -PZDC:ZDCEcalClusters:enableEtaBounds=0 -PZDC:ZDCEcalClusters:energyWeight=log -PZDC:ZDCEcalClusters:input_protoclust_tag=ZDCEcalIslandProtoClusters -PZDC:ZDCEcalClusters:input_simhit_tag=ZDCEcalHits -PZDC:ZDCEcalClusters:logWeightBase=6.2 -PZDC:ZDCEcalClusters:moduleDimZName= -PZDC:ZDCEcalClusters:samplingFraction=1 -PZDC:ZDCEcalIslandProtoClusters:dimScaledLocalDistXY=5,5 -PZDC:ZDCEcalIslandProtoClusters:globalDistEtaPhi= -PZDC:ZDCEcalIslandProtoClusters:globalDistRPhi= -PZDC:ZDCEcalIslandProtoClusters:localDistXY=50,50 -PZDC:ZDCEcalIslandProtoClusters:localDistXZ= -PZDC:ZDCEcalIslandProtoClusters:localDistYZ= -PZDC:ZDCEcalIslandProtoClusters:minClusterCenterEdep=3.0 -PZDC:ZDCEcalIslandProtoClusters:minClusterHitEdep=0.1 -PZDC:ZDCEcalIslandProtoClusters:sectorDist=5 -PZDC:ZDCEcalIslandProtoClusters:splitCluster=1 -PZDC:ZDCEcalTruthClusters:depthCorrection=0 -PZDC:ZDCEcalTruthClusters:enableEtaBounds=0 -PZDC:ZDCEcalTruthClusters:energyWeight=log -PZDC:ZDCEcalTruthClusters:input_protoclust_tag=ZDCEcalTruthProtoClusters -PZDC:ZDCEcalTruthClusters:input_simhit_tag=ZDCEcalHits -PZDC:ZDCEcalTruthClusters:logWeightBase=3.6 -PZDC:ZDCEcalTruthClusters:moduleDimZName= -PZDC:ZDCEcalTruthClusters:samplingFraction=1 -PBTRK:BarrelTrackerRawHit:Threshold=0 -PBTRK:BarrelTrackerRawHit:TimeResolution=8 -PBTRK:BarrelTrackerHit:TimeResolution=8 -PBVTX:BarrelVertexRawHit:Threshold=0 -PBVTX:BarrelVertexRawHit:TimeResolution=8 -PBVTX:BarrelVertexHit:TimeResolution=8 -PECTRK:EndcapTrackerRawHit:Threshold=0 -PECTRK:EndcapTrackerRawHit:TimeResolution=8 -PECTRK:EndcapTrackerHit:TimeResolution=8 -PMPGD:MPGDTrackerRawHit:Threshold=0 -PMPGD:MPGDTrackerRawHit:TimeResolution=8 -PMPGD:MPGDTrackerHit:TimeResolution=8 -PECTOF:TOFEndcapRawHit:Threshold=0 -PECTOF:TOFEndcapRawHit:TimeResolution=0.025 -PECTOF:TOFEndcapTrackerHit:TimeResolution=0.025 -PBTOF:TOFBarrelRawHit:Threshold=0 -PBTOF:TOFBarrelRawHit:TimeResolution=0.025 -PBTOF:TOFBarrelTrackerHit:TimeResolution=0.025 -Ptracking:CentralCKFTrajectories:Chi2CutOff=15 -Ptracking:CentralCKFTrajectories:EtaBins= -Ptracking:CentralCKFTrajectories:NumMeasurementsCutOff=10 -PRPOTS:ForwardRomanPotRawHits:threshold=0.0 -PRPOTS:ForwardRomanPotRawHits:timeResolution=8.0 -PRPOTS:ForwardRomanPotRecHits:time_resolution=0.0 -PRPOTS:ForwardRomanPotParticles:local_x_offset_station_1=-833.3878326 -PRPOTS:ForwardRomanPotParticles:local_x_offset_station_2=-924.342804 -PRPOTS:ForwardRomanPotParticles:local_x_slope_offset=-0.00622147 -PRPOTS:ForwardRomanPotParticles:local_y_slope_offset=-0.0451035 -PRPOTS:ForwardRomanPotParticles:crossingAngle=-0.025 -PRPOTS:ForwardRomanPotParticles:nomMomentum=275.0 -PRPOTS:ForwardRomanPotParticles:m_readout= -PRPOTS:ForwardRomanPotParticles:m_layerField= -PRPOTS:ForwardRomanPotParticles:m_sectorField= -PRPOTS:ForwardRomanPotParticles:m_localDetElement= -PRPOTS:ForwardRomanPotParticles:u_localDetFields= -PReco:GeneratedParticles:MomentumSmearing=0
if [ "$?" -ne "0" ] ; then
  echo "ERROR running eicrecon"
  exit 1
fi
if [ -f jana.dot ] ; then cp jana.dot results/ ; fi

rootls -t ${JUGGLER_REC_FILE_BASE}.tree.edm4eic.root
